package Catalyst::Plugin::Message::SimpleHash;
$Catalyst::Plugin::Message::SimpleHash::VERSION = '0.20181121';
use feature ":5.10";
use Moose::Role;
use Params::Util qw(_HASH);

has messages => (
    traits  => ['Array'],
    is      => 'rw',
    isa     => 'ArrayRef[HashRef]',
    default => sub { [] },
    handles => {
        message        => 'push',
        all_messages   => 'elements',
        count_messages => 'count',
        has_messages   => 'count',
        reset_messages => 'clear',
    },
);

our $flash_key = '_messages';

after prepare_action => sub {
    my $c = shift;

    if ( $c->can('flash') and $c->flash->{$flash_key} ) {
        $c->{messages} = delete $c->flash->{$flash_key};
    }
};

before finalize_session => sub {
    my $c = shift;

    if ($c->has_messages) {
        $c->flash->{$flash_key} = $c->{messages};
        $c->keep_flash($flash_key);
    }
};

around message => sub {
    my ($orig, $c, $message) = (shift, shift, @_);

    my $config             = $c->config->{'Plugin::Message::SimpleHash'};
    my $default_type_value = $config->{default_type_value} || 'success';
    my $message_key        = $config->{message_key}        || 'message';

    unless ( _HASH $message ) {
        $message = { $message_key => $message };
    }
    $message->{type} ||= $default_type_value;

    $c->$orig($message);
};

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Catalyst::Plugin::Message::SimpleHash

=head1 VERSION

version 0.20181121

=head1 AUTHOR

Tomohiro Hosaka

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
